@@ -2,31 +2,6 @@
 #' @description imputes data.frame with mixed variable types using automated
 #'              machine learning (AutoML)
 #'
-#' @importFrom utils setTxtProgressBar txtProgressBar capture.output packageVersion
-#' @importFrom h2o h2o.init as.h2o h2o.automl h2o.predict h2o.ls
-#'             h2o.removeAll h2o.rm h2o.shutdown
-#' @importFrom md.log md.log
-#' @importFrom memuse Sys.meminfo
-#' @importFrom stats var setNames na.omit
-#' @param data a \code{data.frame} or \code{matrix} with missing data to be
-#'             imputed. if \code{load} is provided, this argument will be ignored.
-#' @param m integer, specifying number of multiple imputations. the default value is
-#'          1, carrying out a single imputation.
-#' @param algos character vector, specifying algorithms to be used for missing data
-#'              imputation. the default is 'c("RF", "ELNET", "GBM")', which uses
-#'              Random Forest for a fast initial imputation and then uses ELNET to
-#'              improve the imputation and once ELNET stops improving, attempts using
-#'              "GBM", as long as the 'maxiter' argument is not reached. in other words,
-#'              "mlim" carries out 3 rounds of imputation, which are 1) preimputation with "RF",
-#'              2) imputation with "ELNET", and 3) postimputation with "GBM". the reason for
-#'              this setup is that in general, "RF" is faster than a fine-tuned "ELNET" and
-#'              "ELNET" fine-tunes much faster than "GBM".
-#'
-#'              in addition to these algorithms, \code{"DL"} (Deep Learning) and \code{"XGB"}
-#'              (Extreme Gradient Boosting, only available in Mac OS and Linux) are also
-#'              supported.  "GBM", "DL", "XGB", and "Ensemble" take the full given "tuning_time" (see below) to
-#'        tune the best model for imputing he given variable.
-#'
 # @param preimpute character. specifies the 'primary' procedure of handling the missing
 #                  data, before optimization takes place. the default procedure
 #                  is "rf", which models the missing data with parallel Random Forest
@@ -53,32 +28,40 @@
 #        tune the best model for imputing he given variable. it is advised to use these extensive
 #        algorithms in the process of "postimputation" and let "ELNET" do most of the legwork to save
 #        computational resources.
-# @param postimpute character. specify a vector of algorithms - that are computationally extensive -
-#        to be used to optimize the imputed results. default imputation. possible algorithms are
-#        \code{"GBM"}, \code{"DL"}, \code{"XGB"} (available for Mac and Linux), and \code{"Ensemble"}.
-#        the default value is "GBM". postimputation will only run if maximum iteration limit is not
-#        reached. if you specify more than 1 postimputation algorithm, the "tuning_time" will be
-#        devided between different algorithms, but not necessarily equally. generally, "GBM" and "XGB"
-#        tune faster than "DL", and thus are advised to non-expert users, for general purposes.
-#
-#        postimputation fullz consumes the given "tuning_time" to tune a computationally extensive
-#        algorithm and see whether it outperform "ELNET". this procedure extensively consumes RAM,
-#        so make sure you know the limits of your machine before administering it. otherwise,
-#        set this argument to NULL to deactivate postimputation.
-#' @param preimputed.data data.frame. if you have used another software for missing
-#'                      data imputation, you can still optimize the imputation
-#'                      by handing the data.frame to this argument, which will
-#'                      bypass the "preimpute" procedure.
-#' @param init logical. should h2o Java server be initiated? the default is TRUE.
-#'             however, if the Java server is already running, set this argument
-#'             to FALSE.
-#' @param cpu integer. number of CPUs to be dedicated for the imputation.
-#'                 the default takes all of the available CPUs.
-#' @param ram integer. specifies the maximum size, in Gigabytes, of the
-#'                     memory allocation. by default, all the available memory is
-#'                     used for the imputation.
-#'                     large memory size is particularly advised, especially
-#'                     for multicore processes. the more you give the more you get!
+#' @importFrom utils setTxtProgressBar txtProgressBar capture.output packageVersion
+#' @importFrom h2o h2o.init as.h2o h2o.automl h2o.predict h2o.ls
+#'             h2o.removeAll h2o.rm h2o.shutdown
+#' @importFrom md.log md.log
+#' @importFrom memuse Sys.meminfo
+#' @importFrom stats var setNames na.omit
+#' @importFrom curl curl
+#' @param data a \code{data.frame} (strictly) with missing data to be
+#'             imputed. if \code{'load'} argument is provided, this argument will be ignored.
+#' @param m integer, specifying number of multiple imputations. the default value is
+#'          1, carrying out a single imputation.
+#' @param algos character vector, specifying algorithms to be used for missing data
+#'              imputation. supported algorithms are "ELNET", "RF", "GBM", "DL",
+#'              "XGB", and "Ensemble". if more than one algorithm is specified,
+#'              mlim changes behavior to save on runtime. for example,
+#'              the default is "ELNET", which only uses Elastic Net for the imputation.
+#'              However, 'algos = c("ELNET", "GBM")' will not only use
+#'              ELNET for the initial imputation, but also, uses 'GBM'
+#'              as long as the 'maxiter' argument is not reached or GBM stops
+#'              improving. However, note that by specifying more than one algorithm,
+#'              "mlim" does not fine-tune them all together. Instead, it carries
+#'              out imputation with the first one and when the algorithm stops
+#'              improving, it follows with postimputation, (in this example "GBM")
+#'              to further optimize the imputations. the reason for having
+#'              this setup is that in general, "ELNET" fine-tunes much faster than "GBM",
+#'              "XGB", and "DL".
+#'
+#'              Note that code{"XGB"} is only available in Mac OS and Linux. moreover,
+#'              "GBM", "DL", "XGB", and "Ensemble" take the full given "tuning_time" (see below) to
+#'              tune the best model for imputing he given variable.
+#' @param postimpute logical. if TRUE, mlim uses algorithms rather than 'ELNET' for carrying out
+#'                   postimputation optimization. however, if FALSE, all specified algorihms will
+#'                   be used in the process of 'reimputation' together. the 'Ensemble' algorithm
+#'                   is encouraged when other algorithms are used.
 # @param min_ram character. specifies the minimum size.
 #' @param ignore character vector of column names or index of columns that should
 #'               should be ignored in the process of imputation.
@@ -97,6 +80,16 @@
 #'                   more time in the process of individualized fine-tuning.
 #'                   as a result, the better tuned the model, the more accurate
 #'                   the imputed values are expected to be
+#' @param autobalance logical. if TRUE, binary and multinomial factor variables
+#'                    will be balanced before the imputation to increase the
+#'                    Mean Per Class Error (MPCE) in the process of optimization.
+#'                    if FALSE, MMPCE will be sacrificed for overall accuracy, which
+#'                    is not recommended. in fact, higher overall accuracy does
+#'                    not mean a better imputation as long as minority classes
+#'                    are neglected, which increases the bias in favor of the
+#'                    majority class. if you do not wish to autobalance all the
+#'                    factor variables, you can manually specify the variables
+#'                    that should be balanced using the 'balance' argument (see below)
 #' @param balance character vector, specifying variable names that should be
 #'                balanced before imputation. balancing the prevalence might
 #'                decrease the overall accuracy of the imputation, because it
@@ -119,12 +112,13 @@
 #                    recommended that you set this argument to FALSE.
 #' @param maxiter integer. maximum number of iterations. the default value is \code{15},
 #'        but it can be reduced to \code{3} (not recommended, see below).
-#' @param miniter integer. minimum number of iterations. the default value is
-#'                2.
-#' @param flush logical (experimental). if TRUE, after each model, the server is
-#'              cleaned to retrieve RAM. this feature is in testing mode.
-#' @param cv logical. specify number of k-fold Cross-Validation (CV). values of
-#'               10 or higher are recommended. default is 10.
+# @param miniter integer. minimum number of iterations. the default value is
+#                2.
+# @param flush logical (experimental). if TRUE, after each model, the server is
+#              cleaned to retrieve RAM. this feature is in testing mode and is
+#              currently set to TRUE.
+# @param cv logical. specify number of k-fold Cross-Validation (CV). values of
+#               10 or higher are recommended. default is 10.
 # @param error_metric character. specify the minimum improvement
 #                                  in the estimated error to proceed to the
 #                                  following iteration or stop the imputation.
@@ -139,8 +133,8 @@
 #'                  if the \code{maxiter} is not yet reached. any improvement of imputation
 #'                  is desirable.  however, specifying values above 0 can reduce the number
 #'                  of required iterations at a marginal increase of imputation error.
-#'                  for larger datasets, value of "1e-3" is recommended. note that the
-#'                  best accuracy is reached when this value is equal to zero.
+#'                  for larger datasets, value of "1e-3" is recommended to reduce number
+#'                  of iterations. the default value is '1e-3'.
 #' @param doublecheck logical. default is TRUE (which is conservative). if FALSE, if the estimated
 #'                    imputation error of a variable does not improve, the variable
 #'                    will be not reimputed in the following iterations. in general,
@@ -163,8 +157,6 @@
 #'                       repeating that observation twice in the dataframe.
 #'                       the higher the weight, the more important an observation
 #'                       becomes in the modeling process. the default is NULL.
-#' @param ... Arguments passed to \code{h2o.automl()}.
-#' The following arguments are e.g. incompatible with \code{ranger}: \code{write.forest}, \code{probability}, \code{split.select.weights}, \code{dependent.variable.name}, and \code{classification}.
 #' @param seed integer. specify the random generator seed
 # @param plot logical. If TRUE, estimated error of the imputed dataset is plotted,
 #        showing the reduction in CV error
@@ -179,11 +171,20 @@
 #'               reduced markdown-like report is generated. default is NULL.
 #' @param verbosity character. controls how much information is printed to console.
 #'                  the value can be "warn" (default), "info", "debug", or NULL.
-#' @param shutdown logical. if TRUE, h2o server is closed after the imputation.
-#'                 the default is TRUE
-#' @param sleep integer. number of seconds to wait after each interaction with h2o
-#'              server. the default is 1 second. larger values might be needed
-#'              depending on your computation power or dataset size.
+# @param init logical. should h2o Java server be initiated? the default is TRUE.
+#             however, if the Java server is already running, set this argument
+#'             to FALSE.
+#' @param cpu integer. number of CPUs to be dedicated for the imputation.
+#'                 the default takes all of the available CPUs.
+#' @param ram integer. specifies the maximum size, in Gigabytes, of the
+#'                     memory allocation. by default, all the available memory is
+#'                     used for the imputation.
+#'                     large memory size is particularly advised, especially
+#'                     for multicore processes. the more you give the more you get!
+#' @param preimputed.data data.frame. if you have used another software for missing
+#'                      data imputation, you can still optimize the imputation
+#'                      by handing the data.frame to this argument, which will
+#'                      bypass the "preimpute" procedure.
 #' @param save (NOT YET IMPLEMENTED FOR R). filename. if a filename is specified, an \code{mlim} object is
 #'             saved after the end of each variable imputation. this object not only
 #'             includes the imputed dataframe and estimated cross-validation error, but also
@@ -197,11 +198,19 @@
 #'                 the imputation and is particularly recommended for large datasets
 #'                 or when the user specifies a computationally extensive settings
 #'                 (e.g. specifying several algorithms, increasing tuning time, etc.).
-#' @param force.load (NOT YET IMPLEMENTED FOR R).logical (default is TRUE). if TRUE, when loading the mlim class
-#'                 object, its preserved settings are used for restoring and saving the
-#'                 following itterations. otherwise, if FALSE, the current arguments of
-#'                 mlim are used to overpower the settings of the mlim object. the settings
-#'                 include the full list of the mlim arguments.
+# @param force.load (NOT YET IMPLEMENTED FOR R).logical (default is TRUE). if TRUE, when loading the mlim class
+#                 object, its preserved settings are used for restoring and saving the
+#                 following itterations. otherwise, if FALSE, the current arguments of
+#                 mlim are used to overpower the settings of the mlim object. the settings
+#                 include the full list of the mlim arguments.
+#' @param shutdown logical. if TRUE, h2o server is closed after the imputation.
+#'                 the default is TRUE
+#' @param java character, specifying path to the executable 64bit Java JDK on the
+#'             Microsoft Windows machines, if JDK is installed but the path environment
+#'             variable is not set.
+#' @param ... arguments that are used internally between 'mlim' and 'mlim.postimpute'.
+#'            these arguments are not documented in the help file and are not
+#'            intended to be used by end user.
 #' @return a \code{data.frame}, showing the
 #'         estimated imputation error from the cross validation within the data.frame's
 #'         attribution
@@ -211,49 +220,74 @@
 #'
 #' \donttest{
 #' data(iris)
-#' irisNA <- mlim.na(iris, p = 0.1, stratify = TRUE, seed = 2022)
 #'
-#' # run the default imputation (fastest imputation via 'mlim')
-#' MLIM <- mlim(irisNA)
-#' mlim.error(MLIM, irisNA, iris)
 #'
-#' # run GBM model and allow 15 minutes of tuning for each variable
-#' MLIM <- mlim(irisNA, impute = "GBM", tuning_time=60*15)
-#' mlim.error(MLIM, irisNA, iris)
+#' # add stratified missing observations to the data. to make the example run
+#' # faster, I add NAs only to a single variable.
+#' dfNA <- iris
+#' dfNA$Species <- mlim.na(dfNA$Species, p = 0.1, stratify = TRUE, seed = 2022)
+#'
+#' # run the ELNET single imputation (fastest imputation via 'mlim')
+#' MLIM <- mlim(dfNA, shutdown = FALSE)
+#'
+#' # in single imputation, you can estimate the imputation accuracy via cross validation RMSE
+#' mlim.summarize(MLIM)
+#'
+#' ### or if you want to carry out ELNET multiple imputation with 5 datasets.
+#' ### next, to carry out analysis on the multiple imputation, use the 'mlim.mids' function
+#' ### minimum of 5 datasets
+#' MLIM2 <- mlim(dfNA, m = 5)
+#' mids <- mlim.mids(MLIM2, dfNA)
+#' fit <- with(data=mids, exp=glm(Species ~ Sepal.Length, family = "binomial"))
+#' res <- mice::pool(fit)
+#' summary(res)
+#'
+#' # you can check the accuracy of the imputation, if you have the original dataset
+#' mlim.error(MLIM2, dfNA, iris)
+#
+# ### run GBM, RF, ELNET, and Ensemble algos and allow 60 minutes of tuning for each variable
+# ### this requires a lot of RAM on your machine and a lot of time!
+# # MLIM <- mlim(dfNA, algos = c("GBM", "RF","ELNET","Ensemble"), tuning_time=60*60)
+# # mlim.error(MLIM, dfNA, iris)
+#
+# ### if you have a larger data, there is a few things you can set to make the
+# ### algorithm faster, yet, having only a marginal accuracy reduction as a trade-off
+# # MLIM <- mlim(dfNA, algos = 'ELNET', tolerance = 1e-3, doublecheck = FALSE)
 #' }
 #' @export
 
 
 mlim <- function(data = NULL,
                  m = 1,
-                 algos = c("RF", "ELNET", "GBM"), #preimpute, impute, postimpute
-                 preimputed.data = NULL,
+                 algos = c("ELNET"), #impute, postimpute
+                 postimpute = FALSE,
                  ignore = NULL,
 
                  # computational resources
                  tuning_time = 180,
                  max_models = NULL, # run all that you can
                  maxiter = 10L,
-                 miniter = 2L,
-                 cv = 10L,
+                 #miniter = 2L,
+                 #cv = 10L,
                  #validation = 0,
 
                  matching = "AUTO",    #EXPERIMENTAL
+                 autobalance = TRUE,
                  balance = NULL,       #EXPERIMENTAL
-                 #ignore.rank = FALSE, #EXPERIMENTAL
+                 #ignore.rank = FALSE, #to ignore it, they should make it unordered!
                  weights_column = NULL,
 
-                 # general setup
+                 # report and reproducibility
                  seed = NULL,
                  verbosity = NULL,
                  report = NULL,
 
                  # stopping criteria
-                 tolerance = 0, #1e-3
+                 tolerance = 1e-3,
                  doublecheck = TRUE,
 
                  ## simplify the settings by taking these arguments out
-                 #preimpute = "rf",
+                 #preimpute = "RF",
                  #impute = "AUTO",
                  #postimpute = "AUTO",
                  #error_metric  = "RMSE", #??? mormalize it
@@ -264,25 +298,19 @@ mlim <- function(data = NULL,
                  # setup the h2o cluster
                  cpu = -1,
                  ram = NULL,
-                 flush = FALSE,
-                 init = TRUE,
-                 shutdown = TRUE,
-                 sleep = .5,
+                 #flush = TRUE,
+                 #init = TRUE,
 
                  # NOT YET IMPLEMENTED
+                 preimputed.data = NULL,
                  save = NULL,
                  load = NULL,
-                 force.load = TRUE,
+                 shutdown = TRUE,
+                 java = NULL,
+                 #force.load = TRUE,
                  ...
                  ) {
 
-
-  # initial warnings
-  # ============================================================
-  if (m > 1) {
-    cat("multiple imputation feature is in test-mode and its algorithm can change in the future\n")
-  }
-
   # improvements for the next release
   # ============================================================
   # instead of using all the algorithms at each iteration, add the
@@ -291,7 +319,14 @@ mlim <- function(data = NULL,
   # h2o DRF does not give OOB error, so initial comparison preimputation is not possible
   #    HOWEVER, I can estimate the CV for the preimputation procedure
   #
-  # instead of adding postimpute, extract it from specified algorithms
+  # instead of adding postimpute_algos, extract it from specified algorithms
+
+  # check the ... arguments
+  # ============================================================
+  hidden_args <- c("cv", "init", "flush", "ignore.rank", "sleep")
+  stopifnot(
+    "incompatible '...' arguments" = (names(list(...)) %in% hidden_args)
+  )
 
   # Simplify the syntax by taking arguments that are less relevant to the majority
   # of the users out
@@ -299,109 +334,86 @@ mlim <- function(data = NULL,
   #stopping_metric <- "AUTO"
   #stopping_rounds <- 3
   #stopping_tolerance <- 1e-3
-  MI      <- list()
-  metrics <- NULL
-  error   <- NULL
-  debug   <- FALSE
-  verbose <- 0
-  error_metric  <- "RMSE"
-  ignore.rank <- FALSE #EXPERIMENTAL
+  MI          <- list()
+  bdata       <- NULL
+  metrics     <- NULL
+  error       <- NULL
+  debug       <- FALSE
+  cv          <- threeDots(name = "cv", ..., default = 10L)
+  miniter     <- 2L
+  init        <- threeDots(name = "init", ..., default = TRUE)
+  flush       <- threeDots(name = "flush", ..., default = TRUE)
+  verbose     <- 0
+  error_metric<- "RMSE"
+  preimpute   <- "RF"
+  ignore.rank <- threeDots(name = "ignore.rank", ..., default = FALSE)  #EXPERIMENTAL
+  sleep       <- threeDots(name = "sleep", ..., default = .25)
   set.seed(seed)
 
-  alg <- algoSelector(algos)
-  preimpute <- "RF" #alg$preimpute
-  impute <- alg$impute
-  postimpute <- alg$postimpute
 
 
-
-  #if (is.null(postimpute)) {
-  #  if (impute == "AUTO") postimpute <- "DRF"
-  #  # ...
-  #}
-  #else if (length(postimpute) == 1 & postimpute[1] == "AUTO") {
-  #  #1. check the OS
-  #  #2. make sure GBM has at least 200 non-missing rows
-  #  #3. for smaller datasets, prefer XGB, if OS allows
-  #  OS <- Sys.info()["sysname"]
-  #  if (OS == "Windows") {
-
-  #  }
-  #}
-
-  # feature request:
-  # ??? add arguments in syntaxProcessing to make sure load is well-prepared
-
   # ============================================================
   # ============================================================
   # LOAD SETTINGS FROM mlim class object
   # ============================================================
   # ============================================================
   if (!is.null(load)) {
-    if (inherits(load, "mlim")) stop("'load' must be of class 'mlim'")
+    if (inherits(load, "character")) load <- readRDS(load)
+    if (!inherits(load, "mlim")) stop("loaded object must be of class 'mlim'")
 
     # Data
     # ----------------------------------
-    data        <- load$data
-    dataLast    <- load$dataLast
-    metrics     <- load$metrics
-    mem         <- load$mem
-    orderedCols <- load$orderedCols
+    MI             <- load$MI
+    dataNA         <- load$dataNA
+    data           <- load$data
+    bdata          <- load$bdata
+    dataLast       <- load$dataLast
+    metrics        <- load$metrics
+    mem            <- load$mem
+    orderedCols    <- load$orderedCols
 
     # Loop data
     # ----------------------------------
-    k           <- load$k
-    z           <- load$z
-    X           <- load$X
-    Y           <- load$Y
-    vars2impute <- load$vars2impute
-    FAMILY      <- load$FAMILY
+    m              <- load$m
+    m.it           <- load$m.it
+    k              <- load$k
+    z              <- load$z
+    X              <- load$X
+    Y              <- load$Y
+    vars2impute    <- load$vars2impute
+    FAMILY         <- load$FAMILY
 
     # settings
     # ----------------------------------
-    if (force.load) {
-      impute         <- load$impute
-      ignore         <- load$ignore
-      save           <- load$save
-      maxiter        <- load$maxiter
-      miniter        <- load$miniter
-      cv             <- load$cv
-      tuning_time    <- load$tuning_time
-      max_models     <- load$max_models
-      matching       <- load$matching
-      ignore.rank    <- load$ignore.rank #KEEP IT HIDDEN
-      weights_column <- load$weights_column
-      seed           <- load$seed
-      verbosity      <- load$verbosity
-      verbose        <- load$verbose #KEEP IT HIDDEN
-      debug          <- load$debug   #KEEP IT HIDDEN
-      report         <- load$report
-      flush          <- load$flush
-      error_metric   <- load$error_metric #KEEP IT HIDDEN
-      error          <- load$error  #KEEP IT HIDDEN
-      tolerance      <- load$tolerance
-      cpu            <- load$cpu
-      max_ram        <- load$max_ram
-      min_ram        <- load$min_ram #KEEP IT HIDDEN
-      pkg            <- load$pkg #KEEP IT HIDDEN
-    }
-    else {
-      cat("The following settings are ignored from the loaded 'mlim' object:\n\nn")
-      cat("'impute', 'ignore', 'save', 'maxiter', 'miniter', 'cv', \n")
-      cat("'tuning_time', 'max_models', 'matching', 'weights_column', \n")
-      cat("'seed', 'verbosity', 'report', 'flush', 'tolerance', 'cpu', 'ram'\n")
-
-      synt <- syntaxProcessing(data, preimpute, impute, ram,
-                               matching=matching, miniter, maxiter, max_models,
-                               tuning_time, cv, weights_column,
-                               verbosity, report)
-
-      min_ram <- synt$min_ram
-      max_ram <- synt$max_ram
-      keep_cross_validation_predictions <- synt$keep_cross_validation_predictions
-      verbose <- synt$verbose
-      debug <- synt$debug
-    }
+    ITERATIONVARS  <- load$ITERATIONVARS
+    impute         <- load$impute
+    postimputealgos<- load$postimputealgos
+    autobalance    <- load$autobalance
+    balance        <- load$balance
+    ignore         <- load$ignore
+    save           <- load$save
+    maxiter        <- load$maxiter
+    miniter        <- load$miniter
+    cv             <- load$cv
+    tuning_time    <- load$tuning_time
+    max_models     <- load$max_models
+    matching       <- load$matching
+    ignore.rank    <- load$ignore.rank #KEEP IT HIDDEN
+    weights_column <- load$weights_column
+    seed           <- load$seed
+    verbosity      <- load$verbosity
+    verbose        <- load$verbose #KEEP IT HIDDEN
+    debug          <- load$debug   #KEEP IT HIDDEN
+    report         <- load$report
+    flush          <- load$flush
+    error_metric   <- load$error_metric #KEEP IT HIDDEN
+    error          <- load$error  #KEEP IT HIDDEN
+    tolerance      <- load$tolerance
+    cpu            <- load$cpu
+    max_ram        <- load$max_ram
+    min_ram        <- load$min_ram #KEEP IT HIDDEN
+    keep_cv        <- load$keep_cv
+    pkg            <- load$pkg #KEEP IT HIDDEN
   }
 
   # ============================================================
@@ -410,13 +422,17 @@ mlim <- function(data = NULL,
   # ============================================================
   # ============================================================
   else {
+    alg <- algoSelector(algos, postimpute)
+    # preimpute <- "RF" #alg$preimpute ## for now, make this global
+    impute <- alg$impute
+    postimputealgos <- alg$postimpute
+
     synt <- syntaxProcessing(data, preimpute, impute, ram,
                              matching=matching, miniter, maxiter, max_models,
                              tuning_time, cv, weights_column, verbosity=verbosity, report)
-
     min_ram <- synt$min_ram
     max_ram <- synt$max_ram
-    keep_cross_validation_predictions <- synt$keep_cross_validation_predictions
+    keep_cv <- synt$keep_cross_validation_predictions
     verbose <- synt$verbose
     debug <- synt$debug
   }
@@ -438,17 +454,17 @@ mlim <- function(data = NULL,
            date=TRUE, time=TRUE) #, print=TRUE
   }
 
-
   # Run H2O on the statistics server¤
   # ============================================================
   if (init) {
     #sink(file = report, append = TRUE)
-    #cat("\n") # for Markdown styling
+    #message("\n") # for Markdown styling
     capture.output(connection <- init(nthreads = cpu,
-                       min_mem_size = min_ram,
-                       max_mem_size = max_ram,
-                       ignore_config = TRUE,
-                       report),
+                                      min_mem_size = min_ram,
+                                      max_mem_size = max_ram,
+                                      ignore_config = TRUE,
+                                      java = java,
+                                      report),
                    file = report,
                    append = TRUE)
     #sink()
@@ -468,37 +484,73 @@ mlim <- function(data = NULL,
 
   # Identify variables for imputation and their models' families
   # ============================================================
-  VARS <- selectVariables(data, ignore, verbose, report)
-  dataNA <- VARS$dataNA
-  allPredictors <- VARS$allPredictors
-  vars2impute <- VARS$vars2impute
-  vars2postimpute <- VARS$vars2impute
-  storeVars2impute <- vars2impute
-  X <- VARS$X
+  if (is.null(load)) {
+    VARS <- selectVariables(data, ignore, verbose, report)
+    dataNA <- VARS$dataNA
+    allPredictors <- VARS$allPredictors
+    vars2impute <- VARS$vars2impute
+    vars2postimpute <- VARS$vars2impute
+    storeVars2impute <- vars2impute
+    X <- VARS$X
+    bdata <- NULL
 
-  # if there is only one variable to impute, there is no need to iterate!
-  if (length(vars2impute) == 1) maxiter <- 1
+    # if there is only one variable to impute, there is no need to iterate!
+    if (length(vars2impute) < 1) stop("\nthere is nothing to impute!\n")
+    else if (length(vars2impute) == 1) {
+      if (!is.valid(postimputealgos)) {
+        maxiter <- 1
+      }
+    }
 
-  Features <- checkNconvert(data, vars2impute, ignore,
-                            ignore.rank=ignore.rank, report)
+    # .........................................................
+    # check the variables for compatibility
+    # .........................................................
+    # if preimputed data is provided, take it into consideration!
+    if (!is.null(preimputed.data)) {
 
-  FAMILY<- Features$family
-  data  <- Features$data
-  mem <- Features$mem
-  orderedCols <- Features$orderedCols
+      # if a multiple imputation object is given, take the first dataset
+      # ??? in the future, consider that each of the given datasets can
+      # be fed independently as a separate "m". for now, this is NOT AN
+      # announced feature and thus, just take the first dataset as preimputation
+      if (inherits(preimputed.data, "mlim.mi")) {
+        #preimputed.data <- preimputed.data[[1]]
+        stop("use 'mlim.postimpute' function for postimputing multiple imputation datasets\n")
+      }
 
-  # ??? deactivate "iterate" preimputation, because it's dull!
-  # .........................................................
-  # PREIMPUTATION
-  # .........................................................
-  if (preimpute != "iterate" & is.null(preimputed.data)) {
-    data <- mlim.preimpute(data=data, preimpute=preimpute,
-                           seed = seed, report=report, debug=debug)
+      # if the preimputation was done with mlim, extract the metrics
+      else if (inherits(preimputed.data, "mlim")) {
 
-    # reset the relevant predictors
-    X <- allPredictors
+
+        # remove the NAs of the last imputation and replace them with
+        # the minimum
+        metrics <- getMetrics(preimputed.data)
+      }
+
+      data <- preimputed.data
+
+      # reset the relevant predictors
+      X <- allPredictors
+    }
+
+    Features <- checkNconvert(data, vars2impute, ignore,
+                              ignore.rank=ignore.rank, report)
+
+    FAMILY<- Features$family
+    data  <- Features$data
+    mem <- Features$mem
+    orderedCols <- Features$orderedCols
+
+    # ??? deactivate "iterate" preimputation, because it's dull!
+    # .........................................................
+    # PREIMPUTATION
+    # .........................................................
+    if (preimpute != "iterate" & is.null(preimputed.data)) {
+      data <- mlim.preimpute(data=data, preimpute=preimpute, seed = seed)
+
+      # reset the relevant predictors
+      X <- allPredictors
+    }
   }
-  else if (!is.null(preimputed.data)) data <- preimputed.data
 
   # ............................................................
   # ............................................................
@@ -506,30 +558,34 @@ mlim <- function(data = NULL,
   # ............................................................
   # ............................................................
   if (is.null(load)) {
-    k <- 0L
+    k     <- 1L
+    z     <- 1L
+    m.it  <- 1L
+    MI    <- NULL
     error <- setNames(rep(1, length(vars2impute)), vars2impute)
   }
 
-  for (MIit in 1:m) {
-    dataLast <- iteration_loop(MIit, dataNA, data, boot=MIit>1, #bootstrap if MIit is mroe than 1
+  for (m.it in m.it:m) {
+    dataLast <- iteration_loop(MI, dataNA, data, bdata, boot=m>1,
                                metrics, tolerance, doublecheck,
-                               k, X, Y, z,
+                               m, k, X, Y, z, m.it,
                                # loop data
                                vars2impute, vars2postimpute, storeVars2impute,
-                               allPredictors, preimpute, impute, postimpute,
+                               allPredictors, preimpute, impute, postimputealgos,
                                # settings
                                error_metric, FAMILY=FAMILY, cv, tuning_time,
                                max_models, weights_column,
-                               keep_cross_validation_predictions,
-                               balance, seed, save, flush,
+                               keep_cv,
+                               autobalance, balance, seed, save, flush,
                                verbose, debug, report, sleep,
                                # saving settings
                                mem, orderedCols, ignore, maxiter,
                                miniter, matching, ignore.rank,
                                verbosity, error, cpu, max_ram=max_ram, min_ram=min_ram,
+                               #??? shutdown has to be fixed in future updates
                                shutdown=FALSE, clean = TRUE)
 
-    if (m > 1) MI[[MIit]] <- dataLast
+    if (m > 1) MI[[m.it]] <- dataLast
     else MI <- dataLast
   }
 
